@@ -9,6 +9,12 @@ from nose.plugins.manager import PluginManager
 from nose.plugins.skip import Skip
 from nose import loader
 from nose import suite
+from nose.result import _TextTestResult
+try:
+    # 2.7+
+    from unittest.runner import _WritelnDecorator
+except ImportError:
+    from unittest import _WritelnDecorator
 
 support = os.path.abspath(os.path.join(os.path.dirname(__file__), 'support'))
 
@@ -225,8 +231,8 @@ class TestNoseTestLoader(unittest.TestCase):
         self.assertEqual(m.state, expect, diff(expect, m.state))
 
     def test_fixture_context_multiple_names_some_common_ancestors(self):
-        stream = unittest._WritelnDecorator(StringIO())
-        res = unittest._TextTestResult(stream, 0, 2)
+        stream = _WritelnDecorator(StringIO())
+        res = _TextTestResult(stream, 0, 2)
         wd = os.path.join(support, 'ltfn')
         l = loader.TestLoader(workingDir=wd)
         suite = l.loadTestsFromNames(
@@ -256,8 +262,8 @@ class TestNoseTestLoader(unittest.TestCase):
         self.assertEqual(m.called, expect, diff(expect, m.called))
 
     def test_fixture_context_multiple_names_no_common_ancestors(self):
-        stream = unittest._WritelnDecorator(StringIO())
-        res = unittest._TextTestResult(stream, 0, 2)
+        stream = _WritelnDecorator(StringIO())
+        res = _TextTestResult(stream, 0, 2)
         wd = os.path.join(support, 'ltfn')
         l = loader.TestLoader(workingDir=wd)
         suite = l.loadTestsFromNames(
@@ -336,8 +342,8 @@ class TestNoseTestLoader(unittest.TestCase):
         l = loader.TestLoader(workingDir=ctx)
         suite = l.loadTestsFromName('no_such_module.py')
 
-        res = unittest._TextTestResult(
-            stream=unittest._WritelnDecorator(sys.stdout),
+        res = _TextTestResult(
+            stream=_WritelnDecorator(sys.stdout),
             descriptions=0, verbosity=1)
         suite(res)
 
@@ -353,8 +359,8 @@ class TestNoseTestLoader(unittest.TestCase):
         l = loader.TestLoader(workingDir=ctx)
         suite = l.loadTestsFromName('no_such_module')
 
-        res = unittest._TextTestResult(
-            stream=unittest._WritelnDecorator(sys.stdout),
+        res = _TextTestResult(
+            stream=_WritelnDecorator(sys.stdout),
             descriptions=0, verbosity=1)
         suite(res)
         print res.errors
@@ -370,8 +376,8 @@ class TestNoseTestLoader(unittest.TestCase):
         l = loader.TestLoader(workingDir=ctx)
         suite = l.loadTestsFromName('fred!')
 
-        res = unittest._TextTestResult(
-            stream=unittest._WritelnDecorator(sys.stdout),
+        res = _TextTestResult(
+            stream=_WritelnDecorator(sys.stdout),
             descriptions=0, verbosity=1)
         suite(res)
         print res.errors
@@ -388,13 +394,27 @@ class TestNoseTestLoader(unittest.TestCase):
         gen = os.path.join(support, 'gen')
         l = loader.TestLoader(workingDir=gen)
         suite = l.loadTestsFromName('test')
-        res = unittest._TextTestResult(
-            stream=unittest._WritelnDecorator(sys.stdout),
+        res = _TextTestResult(
+            stream=_WritelnDecorator(sys.stdout),
             descriptions=0, verbosity=1)
         suite(res)
         assert not res.errors
         self.assertEqual(res.testsRun, 5)
 
+    def test_issue_269(self):
+        """Test classes that raise exceptions in __init__ do not stop test run
+        """
+        wdir = os.path.join(support, 'issue269')
+        l = loader.TestLoader(workingDir=wdir)
+        suite = l.loadTestsFromName('test_bad_class')
+        res = _TextTestResult(
+            stream=_WritelnDecorator(sys.stdout),
+            descriptions=0, verbosity=1)
+        suite(res)
+        print res.errors
+        self.assertEqual(len(res.errors), 1)
+        assert 'raise Exception("pow")' in res.errors[0][1]
+        
         
 # used for comparing lists
 def diff(a, b):
