@@ -110,20 +110,22 @@ class TestTestProgram(unittest.TestCase):
                            exit=False)
         res = runner.result
         print stream.getvalue()
+        print "-----"
+        print repr(res)
 
-        # some versions of twisted.trial.unittest.TestCase have
-        # runTest in the base class -- this is wrong! But we have
-        # to deal with it
-        if hasattr(TestCase, 'runTest'):
-            expect = 5
-        else:
-            expect = 4
-        self.assertEqual(res.testsRun, expect,
-                         "Expected to run %s tests, ran %s" %
-                         (expect, res.testsRun))
+        self.assertEqual(res.testsRun, 4,
+                         "Expected to run 4 tests, ran %s" % (res.testsRun,))
         assert not res.wasSuccessful()
         assert len(res.errors) == 1
-        assert len(res.failures) == 2
+
+        # In 12.3, Twisted made their skip functionality match unittests, so the
+        # skipped test is no longer reported as a failure.
+        import twisted
+        v = twisted.version
+        if (v.major, v.minor) >= (12, 3):
+            assert len(res.failures) == 1
+        else:
+            assert len(res.failures) == 2
 
     def test_issue_130(self):
         """Collect and run tests in support/issue130 without error.
