@@ -12,15 +12,19 @@ import logging
 import os
 import sys
 import unittest
-from inspect import isfunction, ismethod
+import types
+from inspect import isfunction
+from nose.pyversion import unbound_method, ismethod
 from nose.case import FunctionTestCase, MethodTestCase
 from nose.failure import Failure
 from nose.config import Config
 from nose.importer import Importer, add_path, remove_path
 from nose.selector import defaultSelector, TestAddress
-from nose.util import cmp_lineno, getpackage, isclass, isgenerator, ispackage, \
-    match_last, resolve_name, transplant_func, transplant_class
+from nose.util import func_lineno, getpackage, isclass, isgenerator, \
+    ispackage, regex_last_key, resolve_name, transplant_func, \
+    transplant_class, test_address
 from nose.suite import ContextSuiteFactory, ContextList, LazySuite
+from nose.pyversion import sort_list, cmp_to_key
 
 
 log = logging.getLogger(__name__)
@@ -89,7 +93,10 @@ class TestLoader(unittest.TestLoader):
         if config.addPaths:
             add_path(workingDir, config)        
         self.suiteClass = ContextSuiteFactory(config=config)
-        unittest.TestLoader.__init__(self)     
+
+        self._visitedPaths = set([])
+
+        unittest.TestLoader.__init__(self)
 
     def getTestCaseNames(self, testCaseClass):
         """Override to select with selector, unless
@@ -97,24 +104,36 @@ class TestLoader(unittest.TestLoader):
         """
         if self.config.getTestCaseNamesCompat:
             return unittest.TestLoader.getTestCaseNames(self, testCaseClass)
-        
+
         def wanted(attr, cls=testCaseClass, sel=self.selector):
             item = getattr(cls, attr, None)
-            if not ismethod(item):
+            if isfunction(item):
+                item = unbound_method(cls, item)
+            elif not ismethod(item):
                 return False
             return sel.wantMethod(item)
+
         cases = filter(wanted, dir(testCaseClass))
-        for base in testCaseClass.__bases__:
-            for case in self.getTestCaseNames(base):
-                if case not in cases:
-                    cases.append(case)
+
         # add runTest if nothing else picked
         if not cases and hasattr(testCaseClass, 'runTest'):
             cases = ['runTest']
         if self.sortTestMethodsUsing:
-            cases.sort(self.sortTestMethodsUsing)
+            sort_list(cases, cmp_to_key(self.sortTestMethodsUsing))
         return cases
 
+    def _haveVisited(self, path):
+        # For cases where path is None, we always pretend we haven't visited
+        # them.
+        if path is None:
+            return False
+
+        return path in self._visitedPaths
+
+    def _addVisitedPath(self, path):
+        if path is not None:
+            self._visitedPaths.add(path)
+
     def loadTestsFromDir(self, path):
         """Load tests from the directory at path. This is a generator
         -- each suite of tests from a module or other file is yielded
@@ -128,7 +147,7 @@ class TestLoader(unittest.TestLoader):
             paths_added = add_path(path, self.config)
 
         entries = os.listdir(path)
-        entries.sort(lambda a, b: match_last(a, b, self.config.testMatch))
+        sort_list(entries, regex_last_key(self.config.testMatch))
         for entry in entries:
             # this hard-coded initial-dot test will be removed:
             # http://code.google.com/p/python-nose/issues/detail?id=82
@@ -149,7 +168,14 @@ class TestLoader(unittest.TestLoader):
                         continue
                     wanted = self.selector.wantDirectory(entry_path)
             is_package = ispackage(entry_path)
-            if wanted:
+
+            # Python 3.3 now implements PEP 420: Implicit Namespace Packages.
+            # As a result, it's now possible that parent paths that have a
+            # segment with the same basename as our package ends up
+            # in module.__path__.  So we have to keep track of what we've
+            # visited, and not-revisit them again.
+            if wanted and not self._haveVisited(entry_path):
+                self._addVisitedPath(entry_path)
                 if is_file:
                     plugins.beforeContext()
                     if entry.endswith('.py'):
@@ -181,7 +207,8 @@ class TestLoader(unittest.TestLoader):
         
         # pop paths
         if self.config.addPaths:
-            map(remove_path, paths_added)
+            for p in paths_added:
+              remove_path(p)
         plugins.afterDirectory(path)
 
     def loadTestsFromFile(self, filename):
@@ -208,7 +235,9 @@ class TestLoader(unittest.TestLoader):
             raise
         except:
             exc = sys.exc_info()
-            return self.suiteClass([Failure(*exc)])
+            return self.suiteClass(
+                [Failure(exc[0], exc[1], exc[2],
+                         address=(filename, None, None))])
 
     def loadTestsFromGenerator(self, generator, module):
         """Lazy-load tests from a generator function. The generator function
@@ -218,15 +247,19 @@ class TestLoader(unittest.TestLoader):
         * a function name resolvable within the same module
         """
         def generate(g=generator, m=module):
-            for test in g():
-                try:
-                    test_func, arg = (test[0], test[1:])
-                except ValueError:
-                    test_func, arg = test[0], tuple()
-                if not callable(test_func):
-                    test_func = getattr(m, test_func)
-                yield FunctionTestCase(test_func, arg=arg, descriptor=g)
-        return self.suiteClass(generate)
+            try:
+                for test in g():
+                    test_func, arg = self.parseGeneratedTest(test)
+                    if not callable(test_func):
+                        test_func = getattr(m, test_func)
+                    yield FunctionTestCase(test_func, arg=arg, descriptor=g)
+            except KeyboardInterrupt:
+                raise
+            except:
+                exc = sys.exc_info()
+                yield Failure(exc[0], exc[1], exc[2],
+                              address=test_address(generator))
+        return self.suiteClass(generate, context=generator, can_split=False)
 
     def loadTestsFromGeneratorMethod(self, generator, cls):
         """Lazy-load tests from a generator method.
@@ -240,33 +273,39 @@ class TestLoader(unittest.TestLoader):
         """
         # convert the unbound generator method
         # into a bound method so it can be called below
-        cls = generator.im_class
+        if hasattr(generator, 'im_class'):
+            cls = generator.im_class
         inst = cls()
         method = generator.__name__
         generator = getattr(inst, method)
 
         def generate(g=generator, c=cls):
-            for test in g():
-                try:
-                    test_func, arg = (test[0], test[1:])
-                except ValueError:
-                    test_func, arg = test[0], tuple()
-                if not callable(test_func):
-                    test_func = getattr(c, test_func)
-                if ismethod(test_func):
-                    yield MethodTestCase(test_func, arg=arg, descriptor=g)
-                elif isfunction(test_func):
-                    # In this case we're forcing the 'MethodTestCase'
-                    # to run the inline function as its test call,
-                    # but using the generator method as the 'method of
-                    # record' (so no need to pass it as the descriptor)
-                    yield MethodTestCase(g, test=test_func, arg=arg)
-                else:
-                    yield Failure(TypeError,
-                                  "%s is not a function or method" % test_func)
-        return self.suiteClass(generate)
+            try:
+                for test in g():
+                    test_func, arg = self.parseGeneratedTest(test)
+                    if not callable(test_func):
+                        test_func = unbound_method(c, getattr(c, test_func))
+                    if ismethod(test_func):
+                        yield MethodTestCase(test_func, arg=arg, descriptor=g)
+                    elif isfunction(test_func):
+                        # In this case we're forcing the 'MethodTestCase'
+                        # to run the inline function as its test call,
+                        # but using the generator method as the 'method of
+                        # record' (so no need to pass it as the descriptor)
+                        yield MethodTestCase(g, test=test_func, arg=arg)
+                    else:
+                        yield Failure(
+                            TypeError,
+                            "%s is not a function or method" % test_func)
+            except KeyboardInterrupt:
+                raise
+            except:
+                exc = sys.exc_info()
+                yield Failure(exc[0], exc[1], exc[2],
+                              address=test_address(generator))
+        return self.suiteClass(generate, context=generator, can_split=False)
 
-    def loadTestsFromModule(self, module, discovered=False):
+    def loadTestsFromModule(self, module, path=None, discovered=False):
         """Load all tests from module and return a suite containing
         them. If the module has been discovered and is not test-like,
         the suite will be empty by default, though plugins may add
@@ -288,19 +327,26 @@ class TestLoader(unittest.TestLoader):
                         test_classes.append(test)
                 elif isfunction(test) and self.selector.wantFunction(test):
                     test_funcs.append(test)
-            test_classes.sort(lambda a, b: cmp(a.__name__, b.__name__))
-            test_funcs.sort(cmp_lineno)
+            sort_list(test_classes, lambda x: x.__name__)
+            sort_list(test_funcs, func_lineno)
             tests = map(lambda t: self.makeTest(t, parent=module),
                         test_classes + test_funcs)
 
         # Now, descend into packages
         # FIXME can or should this be lazy?
         # is this syntax 2.2 compatible?
-        paths = getattr(module, '__path__', [])
-        for path in paths:
-            tests.extend(self.loadTestsFromDir(path))
+        module_paths = getattr(module, '__path__', [])
+        if path:
+            path = os.path.realpath(path)
+        for module_path in module_paths:
+            log.debug("Load tests from module path %s?", module_path)
+            log.debug("path: %s os.path.realpath(%s): %s",
+                      path, module_path, os.path.realpath(module_path))
+            if (self.config.traverseNamespace or not path) or \
+                    os.path.realpath(module_path).startswith(path):
+                tests.extend(self.loadTestsFromDir(module_path))
             
-        for test in self.config.plugins.loadTestsFromModule(module):
+        for test in self.config.plugins.loadTestsFromModule(module, path):
             tests.append(test)
 
         return self.suiteClass(ContextList(tests, context=module))
@@ -337,7 +383,8 @@ class TestLoader(unittest.TestLoader):
                 name = addr.call
             parent, obj = self.resolve(name, module)
             if (isclass(parent)
-                and getattr(parent, '__module__', None) != module.__name__):
+                and getattr(parent, '__module__', None) != module.__name__
+                and not isinstance(obj, Failure)):
                 parent = transplant_class(parent, module.__name__)
                 obj = getattr(parent, obj.__name__)
             log.debug("parent %s obj %s module %s", parent, obj, module)
@@ -369,12 +416,14 @@ class TestLoader(unittest.TestLoader):
                     raise
                 except:
                     exc = sys.exc_info()
-                    return suite([Failure(*exc)])
+                    return suite([Failure(exc[0], exc[1], exc[2],
+                                          address=addr.totuple())])
                 if addr.call:
                     return self.loadTestsFromName(addr.call, module)
                 else:
                     return self.loadTestsFromModule(
-                        module, discovered=discovered)
+                        module, addr.filename,
+                        discovered=discovered)
             elif addr.filename:
                 path = addr.filename
                 if addr.call:
@@ -384,7 +433,8 @@ class TestLoader(unittest.TestLoader):
                             Failure(ValueError,
                                     "Can't find callable %s in file %s: "
                                     "file is not a python module" %
-                                    (addr.call, path))])
+                                    (addr.call, path),
+                                    address=addr.totuple())])
                     return self.loadTestsFromName(addr.call, module=package)
                 else:
                     if op_isdir(path):
@@ -400,12 +450,14 @@ class TestLoader(unittest.TestLoader):
                         return self.loadTestsFromFile(path)
                     else:
                         return suite([
-                                Failure(OSError, "No such file %s" % path)])
+                                Failure(OSError, "No such file %s" % path,
+                                        address=addr.totuple())])
             else:
                 # just a function? what to do? I think it can only be
                 # handled when module is not None
                 return suite([
-                    Failure(ValueError, "Unresolvable test name %s" % name)])
+                    Failure(ValueError, "Unresolvable test name %s" % name,
+                            address=addr.totuple())])
 
     def loadTestsFromNames(self, names, module=None):
         """Load tests from all names, returning a suite containing all
@@ -444,11 +496,13 @@ class TestLoader(unittest.TestLoader):
 
         In this case, we can't depend on the class's `__init__` taking method
         name arguments, so we have to compose a MethodTestCase for each
-        method in the class that looks testlike.        
+        method in the class that looks testlike.
         """
         def wanted(attr, cls=cls, sel=self.selector):
             item = getattr(cls, attr, None)
-            if not ismethod(item):
+            if isfunction(item):
+                item = unbound_method(cls, item)
+            elif not ismethod(item):
                 return False
             return sel.wantMethod(item)
         cases = [self.makeTest(getattr(cls, case), cls)
@@ -458,10 +512,31 @@ class TestLoader(unittest.TestLoader):
         return self.suiteClass(ContextList(cases, context=cls))
 
     def makeTest(self, obj, parent=None):
+        try:
+            return self._makeTest(obj, parent)
+        except (KeyboardInterrupt, SystemExit):
+            raise
+        except:
+            exc = sys.exc_info()
+            try:
+                addr = test_address(obj)
+            except KeyboardInterrupt:
+                raise
+            except:
+                addr = None
+            return Failure(exc[0], exc[1], exc[2], address=addr)
+    
+    def _makeTest(self, obj, parent=None):
         """Given a test object and its parent, return a test case
         or test suite.
         """
         plug_tests = []
+        try:
+            addr = test_address(obj)
+        except KeyboardInterrupt:
+            raise
+        except:
+            addr = None
         for test in self.config.plugins.makeTest(obj, parent):
             plug_tests.append(test)
         # TODO: is this try/except needed?
@@ -471,8 +546,14 @@ class TestLoader(unittest.TestLoader):
         except (KeyboardInterrupt, SystemExit):
             raise
         except:
-            return Failure(*sys.exc_info())
+            exc = sys.exc_info()
+            return Failure(exc[0], exc[1], exc[2], address=addr)
         
+        if isfunction(obj) and parent and not isinstance(parent, types.ModuleType):
+	    # This is a Python 3.x 'unbound method'.  Wrap it with its
+	    # associated class..
+            obj = unbound_method(parent, obj)
+
         if isinstance(obj, unittest.TestCase):
             return obj
         elif isclass(obj):
@@ -501,7 +582,8 @@ class TestLoader(unittest.TestLoader):
                 return FunctionTestCase(obj)
         else:
             return Failure(TypeError,
-                           "Can't make a test from %s" % obj)
+                           "Can't make a test from %s" % obj,
+                           address=addr)
 
     def resolve(self, name, module):
         """Resolve name within module
@@ -515,5 +597,20 @@ class TestLoader(unittest.TestLoader):
             obj = Failure(ValueError, "No such test %s" % name)
         return parent, obj
 
+    def parseGeneratedTest(self, test):
+        """Given the yield value of a test generator, return a func and args.
+
+        This is used in the two loadTestsFromGenerator* methods.
+
+        """
+        if not isinstance(test, tuple):         # yield test
+            test_func, arg = (test, tuple())
+        elif len(test) == 1:                    # yield (test,)
+            test_func, arg = (test[0], tuple())
+        else:                                   # yield test, foo, bar, ...
+            assert len(test) > 1 # sanity check
+            test_func, arg = (test[0], test[1:])
+        return test_func, arg
+
 defaultTestLoader = TestLoader
 
